@@ -443,9 +443,13 @@ static int Fprofile_FunctionTrace(PyObject* obj, PyFrameObject* frame, int what,
             mpack_write_u32(writer, tsc.tv_nsec);
             mpack_finish_array(writer);
 
-            mpack_write_cstr(writer, PyUnicode_AsUTF8(frame->f_code->co_name));
-            mpack_write_cstr(writer, PyUnicode_AsUTF8(frame->f_code->co_filename));
-            mpack_write_u32(writer, frame->f_lineno);
+            {
+                PyObject* name     = frame->f_code->co_name;
+                PyObject* filename = frame->f_code->co_filename;
+                mpack_write_cstr(writer, name != NULL ? PyUnicode_AsUTF8(name) : "NULL");
+                mpack_write_cstr(writer, filename != NULL ? PyUnicode_AsUTF8(filename) : "NULL");
+                mpack_write_u32(writer, frame->f_lineno);
+            }
             mpack_finish_array(writer);
             break;
         case PyTrace_RETURN:
@@ -457,29 +461,13 @@ static int Fprofile_FunctionTrace(PyObject* obj, PyFrameObject* frame, int what,
             mpack_write_u32(writer, tsc.tv_nsec);
             mpack_finish_array(writer);
 
-            mpack_write_cstr(writer, PyUnicode_AsUTF8(frame->f_code->co_name));
+            {
+                PyObject* name = frame->f_code->co_name;
+                mpack_write_cstr(writer, name != NULL ? PyUnicode_AsUTF8(name) : "NULL");
+            }
             mpack_finish_array(writer);
             break;
         case PyTrace_C_CALL: {
-            // Attempt to determine what module/class this function belongs to.
-            PyObject*   self        = fn->m_self;
-            PyObject*   module      = fn->m_module;
-            const char* module_name = NULL;
-
-            // Check if we belong to a module, and if not we must be a method.
-            // We do this order to avoid finding that the object we belong to
-            // is of type module.
-            if (module != NULL) {
-                if (PyModule_Check(module)) {
-                    module_name = PyModule_GetName(module);
-                } else if (PyUnicode_Check(module)) {
-                    module_name = PyUnicode_AsUTF8(module);
-                }
-            } else if (self != NULL) {
-                // This is a method call on a class.
-                module_name = self->ob_type->tp_name;
-            }
-
             mpack_start_array(writer, 4);
             mpack_write_cstr(writer, "NativeCall");
 
@@ -488,8 +476,31 @@ static int Fprofile_FunctionTrace(PyObject* obj, PyFrameObject* frame, int what,
             mpack_write_u32(writer, tsc.tv_nsec);
             mpack_finish_array(writer);
 
-            mpack_write_cstr(writer, fn->m_ml->ml_name);
-            mpack_write_cstr(writer, module_name);
+            {
+                // Attempt to determine what module/class this function belongs
+                // to.
+                PyObject*   self        = fn->m_self;
+                PyObject*   module      = fn->m_module;
+                const char* name        = fn->m_ml->ml_name;
+                const char* module_name = NULL;
+
+                // Check if we belong to a module, and if not we must be a
+                // method.  We do this order to avoid finding that the object
+                // we belong to is of type module.
+                if (module != NULL) {
+                    if (PyModule_Check(module)) {
+                        module_name = PyModule_GetName(module);
+                    } else if (PyUnicode_Check(module)) {
+                        module_name = PyUnicode_AsUTF8(module);
+                    }
+                } else if (self != NULL) {
+                    // This is a method call on a class.
+                    module_name = self->ob_type->tp_name;
+                }
+
+                mpack_write_cstr(writer, name != NULL ? name : "NULL");
+                mpack_write_cstr(writer, module_name != NULL ? module_name : "NULL");
+            }
             mpack_finish_array(writer);
             break;
         }
@@ -502,7 +513,10 @@ static int Fprofile_FunctionTrace(PyObject* obj, PyFrameObject* frame, int what,
             mpack_write_u32(writer, tsc.tv_nsec);
             mpack_finish_array(writer);
 
-            mpack_write_cstr(writer, fn->m_ml->ml_name);
+            {
+                const char* name = fn->m_ml->ml_name;
+                mpack_write_cstr(writer, name != NULL ? name : "NULL");
+            }
             mpack_finish_array(writer);
             break;
         default:
